import argparse
from .components import *
from typing import List

__version__ = '1.0'


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('-s', '--submission_dir', help='Path to submision directory', type=str, default='./submissions')
    parser.add_argument('-o', '--output_dir', help='Path to output directory', type=str, default='./output')
    parser.add_argument('--report_name', help='Path to report, will generate <REPORT>.csv|md', type=str, default='REPORT')
    parser.add_argument('-c',
                        '--command',
                        help='Build command',
                        type=str,
                        default='g++ ./src/*.(c|cpp) -I ./include -o main -Wall -g -std=c++14')
    parser.add_argument('--keep_output', action='store_true', default=False, help='Keep output files, do not extract')
    parser.add_argument('--keep_file_structure', action='store_true', default=False, help='Keep structure of source files')
    parser.add_argument('-a',
                        '--apps',
                        type=str,
                        default='filter,builder,reporter,grader',
                        help='Apps to start, defualt to all(filter,builder,reporter,grader)')
    args = parser.parse_args()

    apps: List[str] = args.apps.split(',')
    # Copy files to output_dir
    if 'filter' in apps:
        App1 = AutoFilter(args)
        App1()

    # Build submissions
    if 'builder' in apps:
        App2 = AutoBuilder(args)
        App2()

    # Generate report
    if 'reporter' in apps:
        App3 = AutoReporter(args)
        App3(App1.failed_targets, App2.compiler_output)

    if 'grader' in apps:
        # Grading
        cmd: str = input('[ Info ] Start grading? [y/N]:')
        if cmd == 'y' or cmd == 'Y':
            cmd = input('[ Info ] Save report to? (default: ./GRADES.csv): ')
            path_to_report: str = cmd if len(cmd) > 0 else './GRADES.csv'
            cmd = input('[ Info ] Grading dimensions? (default: Answer Interface):')
            dimension: List[str] = cmd.split(' ') if len(cmd) > 0 else ['Answer', 'Interface']
            App4 = ManualGrader(args.output_dir, path_to_report, dimension)
            App4()


if __name__ == '__main__':
    main()


import os
import os.path as osp
import glob2
from typing import List
import argparse
import difflib
from io import StringIO
import coloredlogs
import logging
import tqdm
import re

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')

DEFAULT_IGNORE_PATTERNS = r'.*CMakeCXXCompilerId.cpp|.*CMakeCCompilerId.c|\.vscode|\.idea|\.git|\.gitignore'


def find_c_source_files(base_dir: str, ext: List[str]) -> List[str]:
    """Find C Source Files in the current directory.

    Args:
        base_dir (str): Base directory
        ext (List[str]): Extension

    Returns:
        List[str]: File names
    """
    return sum([glob2.glob(osp.join(base_dir, '**', '*.{}'.format(e)), recursive=True) for e in ext], [])
    # return sum(*[list(map(lambda x: x.name,Path(base_dir).rglob('*.{}'.format(e)))) for e in ext],[])


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--base_dir', type=str, default='./')
    parser.add_argument('--ext', type=str,
                        default='c,cpp,h,hpp,cc,cxx,c++,h++,hxx,hh')
    parser.add_argument('--threshold', type=int, default=100)
    parser.add_argument('--output_dir', type=str, default='./antiplag_output')
    parser.add_argument('--use_basename', type=bool, default=False)
    parser.add_argument('--path_depth', type=int, default=2)
    parser.add_argument('--ignore_pattern', type=str, default='\.\.')
    # parser.add_argument('--debug', action='store_true')
    # parser.add_argument('--verbose', action='store_true')
    # parser.add_argument('--quiet', action='store_true')
    args = parser.parse_args()
    args.ext = args.ext.split(',')
    return args


def read_files(files: List[str]) -> List[str]:
    """Read files.

    Args:
        files (List[str]): File names

    Returns:
        List[str]: File contents
    """
    return {f: open(f).read() for f in files}


def html_comp(in_lines1: list, in_lines2: list, out_file: str):
    d = difflib.HtmlDiff()
    result = d.make_file(in_lines1, in_lines2)
    with open(out_file, 'w') as f:
        f.writelines(result)


def context_comp(in_lines1: list, in_lines2: list, out_file: str = None):
    d = difflib.context_diff(in_lines1, in_lines2)
    buf = StringIO()
    buf.writelines(d)
    if out_file is not None and out_file != '':
        with open(out_file, 'w') as f:
            f.writelines(d)
    return len(buf.getvalue()), buf


def main():
    args = parse_args()

    # if args.debug:
    #     pass
    #     args.base_dir = './PermutationOut'
    #     args.ext = ['cpp']

    if not osp.exists(args.output_dir):
        logger.info('Creating output directory: {}'.format(args.output_dir))
        os.makedirs(args.output_dir)

    logging.info(f"Processing files in {args.base_dir}")
    list_of_files = find_c_source_files(args.base_dir, ext=args.ext)
    logging.info(f"Reading {len(list_of_files)} files")
    file_contents = read_files(list_of_files)
    num_files = len(file_contents)

    with tqdm.tqdm(total=num_files) as pbar:
        for index1 in range(num_files):
            pbar.set_description(f"Processing file {list_of_files[index1]}")
            for index2 in range(index1+1, num_files):
                file1 = list_of_files[index1]
                file2 = list_of_files[index2]
                if (not len(re.findall(DEFAULT_IGNORE_PATTERNS, file1)) > 0) and (not len(re.findall(DEFAULT_IGNORE_PATTERNS, file2)) > 0):
                    if (not len(re.findall(args.ignore_pattern, file1)) > 0) and (not len(re.findall(args.ignore_pattern, file2)) > 0):
                        num_chars, _ = context_comp(file_contents[file1].split(
                            '\n'), file_contents[file2].split('\n'))

                        if num_chars < args.threshold:
                            logging.warning('{} and {} are {} chars different'.format(
                                file1, file2, num_chars))
                            if args.use_basename:
                                report_name = osp.join(
                                    args.output_dir, '{}-{}.html'.format(osp.basename(file1), osp.basename(file2)))
                            else:
                                report_name = osp.join(args.output_dir, '.'.join(file1.split(
                                    os.sep)[-args.path_depth:]) + '-' + '.'.join(file2.split(os.sep)[-args.path_depth:]) + '.html')
                            html_comp(file_contents[file1].split('\n'),
                                      file_contents[file2].split('\n'),
                                      report_name)
            pbar.update()


if __name__ == '__main__':
    main()



import datetime
from typing import Dict, Tuple
import uuid
import os
import logging
import coloredlogs

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')
class AutoReporter:
    """Creating a report
    """

    def __init__(self, args):
        self.unique_id = str(uuid.uuid1()).split('-')[0]
        self.report_md_path = os.path.join(os.getcwd(), args.report_name + '-' + self.unique_id + '.md')
        self.report_csv_path = os.path.join(os.getcwd(), args.report_name + '-' + self.unique_id + '.csv')

    def _gen_markdown(self, failed_targets: Dict[str, str], compiler_output: Dict[str, Tuple[int, str]]):
        """Generate markdown file

        Args:
            failed_targets (Dict[str, str]): Failed targets
            compiler_output (Dict[str, Tuple[int, str]]): Compiler output

        """
        logging.info("[ Info ] Generating report at {}".format(self.report_md_path))
        with open(self.report_md_path, 'w') as f:
            f.writelines(['# Grader report\n\n',
                          'Time: {}\n'.format(
                              datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')),
                          'Total Submissions: {}\n'.format(
                              len(compiler_output.keys())),
                          '\n## Unprocessed submissions:\n\n'])
            f.writelines(['**{}**\n\n```\n{}\n```\n\n'.format(target,
                                                              failed_targets[target]) for target in failed_targets.keys()])
            f.writelines(['\n## Compiler summary\n\n'])
            f.writelines(['|  name  |  status  |\n', '| ------ | ------- |\n'])
            f.writelines(['| **{}** |    {}    |\n'.format(task_name, str(
                compiler_output[task_name][0] == 0)) for task_name in sorted(compiler_output.keys())])
            f.writelines(['\n## Compiler output\n\n'])
            f.writelines(['**{}**: {}\n\n```\n{}\n```\n\n'.format(task_name, str(compiler_output[task_name]
                                                                                 [0] == 0), compiler_output[task_name][1]) for task_name in sorted(compiler_output.keys())])

    def _gen_csv(self, failed_targets: Dict[str, str], compiler_output: Dict[str, Tuple[int, str]]):
        """Generate CSV report

        Args:
            failed_targets (Dict[str, str]): Failed targets
            compiler_output (Dict[str, Tuple[int, str]]): Compiler output
        """
        logging.info("[ Info ] Generating report(csv) at {}".format(self.report_csv_path))
        with open(self.report_csv_path, 'w') as f:
            f.writelines(['name,status,\n'])
            f.writelines(['{},{},\n'.format(task_name, int(compiler_output[task_name][0] == 0)) for task_name in sorted(compiler_output.keys())])

    def run(self, *args, **kwargs):
        # Generate Markdown report
        self._gen_markdown(*args, **kwargs)

        # Generate CSV report
        self._gen_csv(*args, **kwargs)

    def __call__(self, failed_targets: Dict[str, str], compiler_output: Dict[str, Tuple[int, str]]):
        self.run(failed_targets, compiler_output)



import os
import shutil
import zipfile
import rarfile
from tqdm import tqdm
from typing import Dict, List
import logging
import coloredlogs

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')

def canvas_extract_name(target_name: str) -> str:
    """ Extract name from target -> 5xxxxxxxxxxxNAME


    Args:
        target_name (str): Example: 5xxxxxxxxxxxNAME_*_*_*_.zip

    Returns:
        str: Example 5xxxxxxxxxxxNAME
    """
    return target_name.split('_')[0]

def canvas_remove_prefix(path_to_file: str) -> str:
    """Remove canvas prefix of file

    Args:
        path_to_file (str): 5xxxxxxxxxxxNAME_*_*_filename.cpp 

    Returns:
        str: filename.zip
    """

    return ''.join(os.path.basename(path_to_file).split('_')[3:])

def canvas_test_submission_format(path_to_dir: str) -> List[int]:
    """Check if the submission is of correct format

    Args:
        path_to_dir (str): path to submission

    Returns:
        List[int]: [0,0] -> incorrec, [1,0] -> correct, [1,1] -> correct and the submission is included in another folder
    """
    content_list: List[str] = [f for f in os.listdir(
        path_to_dir) if ((not f.startswith('.')) and (not f.startswith('__MACOSX')))]
    ret: List[int] = [0,0]
    # If the zip file contains only one folder, step in
    if len(content_list) == 1 and os.path.isdir(os.path.join(path_to_dir, content_list[0])):
        content_list = [f for f in os.listdir(os.path.join(
            path_to_dir, content_list[0])) if not f.startswith('.')]
        ret[0] = 1

    if 'CMakeLists.txt' in content_list or 'Makefile' in content_list:
        ret[1] = 1
        return ret

    return ret

class AutoFilter:
    """Filter files
    """

    def __init__(self, args):
        self.submission_dir: str = args.submission_dir  # Submission directory
        # Example:
        # Submission director
        # ├── 5xxxxxxxxxxxNAME_*_*_*.zip
        # ├── ...
        # └── ...

        self.output_dir: str = args.output_dir  # Output directory
        # Example:
        # Output director
        # ├── 5xxxxxxxxxxxNAME
        #     ├── include
        #     ├── src
        #     └── CMakeLists.txt
        # ├── ...
        # └── ...

        # student_name -> [files] mapping
        self.target_mapping: Dict[str, List[str]] = dict()
        # file -> Exception mapping
        self.failed_targets: Dict[str, str] = dict()
        self.keep_output = args.keep_output  # Whether to keep output file
        self.keep_file_structure = args.keep_file_structure

        self._extract_name = canvas_extract_name
        self._remove_prefix = canvas_remove_prefix
        self._test_submission_format = canvas_test_submission_format

    def _create_output_dir(self):
        """Create the output directory, will override the directory
        """

        if os.path.exists(self.output_dir):
            if not self.keep_output:
                logging.info("Output folder exists, override")
                shutil.rmtree(self.output_dir)
                os.mkdir(self.output_dir)
            else:
                logging.info("keep_output==True, keep")
                pass
        else:
            os.mkdir(self.output_dir)

    def _preprocess_zip(self, path_to_zipfile: str):
        """Extract zip file to their related directory
        /path/5xxxxxxxxxxxNAME_*_*_*_.zip will be extract to /path/5xxxxxxxxxxxNAME_*_*_*_(dir)

        Args:
            path_to_zipfile (str): path to strfile
        """

        dir_related: str = os.path.splitext(
            os.path.basename(path_to_zipfile))[0]

        if os.path.exists(dir_related):
            shutil.rmtree(dir_related)
            os.mkdir(dir_related)
        f = zipfile.ZipFile(path_to_zipfile)
        try:
            if not os.path.exists(os.path.join(self.submission_dir, dir_related)):
                f.extractall(os.path.join(self.submission_dir, dir_related))
            else:
                logging.warning("{path_to_zipfile} is uncompressed manually")
        except Exception as err:
            logging.warning("{path_to_zipfile} could not be processed")
            shutil.rmtree(os.path.join(self.submission_dir, dir_related))
            self.failed_targets[path_to_zipfile] = err

    def _preprocess_rar(self, path_to_rarfile):
        """Extract rar file to their related directory

        Args:
            path_to_rarfile (str): Path to rarfile

        Bug:
            Usually dont work because for compatibility issues
        """

        dir_related: str = os.path.splitext(
            os.path.basename(path_to_rarfile))[0]

        if os.path.exists(dir_related):
            shutil.rmtree(dir_related)
            os.mkdir(dir_related)
        f = rarfile.RarFile(path_to_rarfile)
        try:
            f.extractall(os.path.join(self.submission_dir, dir_related))
        except Exception as err:
            logging.warning("{path_to_rarfile} could not be processed".format())
            shutil.rmtree(os.path.join(self.submission_dir, dir_related))
            self.failed_targets[path_to_rarfile] = err

    def _preprocess_tar(self, path_to_tarball: str):
        raise NotImplementedError

    def _preprocess(self):
        """Extract all zip/rar files
        """

        target_list: List[str] = [f for f in os.listdir(
            self.submission_dir) if not f.startswith('.')]
        for target in target_list:
            target_path = os.path.join(self.submission_dir, target)
            if os.path.isfile(target_path):
                if zipfile.is_zipfile(target_path):
                    self._preprocess_zip(target_path)
                if rarfile.is_rarfile(target_path):
                    self._preprocess_rar(target_path)

    def _map_submission(self):
        """Map submission to student names(IDs)
        """

        target_list: List[str] = [f for f in os.listdir(
            self.submission_dir) if not f.startswith('.')]
        for target in target_list:
            ext = os.path.splitext(target)[-1]

            # Ignore hidden files, compressed files
            if target[0] == '.' or ext in ['.rar', '.zip']:
                continue

            # Extract names from submission
            name = target.split('_')[0]

            if name in self.target_mapping.keys():
                self.target_mapping[name].append(
                    os.path.join(self.submission_dir, target))
            else:
                self.target_mapping[name] = [
                    os.path.join(self.submission_dir, target)]

    def _create_alldir(self):
        """Prepare output directory for everyone

        """
        for name in self.target_mapping.keys():
            if not os.path.exists(os.path.join(self.output_dir, name)):
                os.mkdir(os.path.join(self.output_dir, name))
                os.mkdir(os.path.join(self.output_dir, name, 'src'))
                os.mkdir(os.path.join(self.output_dir, name, 'include'))

    def _clean_dir(self, path_to_dir: str):
        """Clean a directory by recreation

        Args:
            path_to_dir (str): Output directory
        """
        shutil.rmtree(path_to_dir)
        os.mkdir(path_to_dir)

    def _filter_process_plain(self, student_name: str, path_to_file: str, remove_prefix: bool = False):
        """Process uncompressed files

        Args:
            student_name (str): student's name 5xxxxxxxxxxxNAME
            path_to_file (str): path to file
            remove_prefix (bool, optional): if the canvas prefix exist (and should be removed). Defaults to False.

        Returns:
            bool: if the file is filtered out
        """
        ext: str = os.path.splitext(path_to_file)[-1]

        # Match extension
        if remove_prefix:
            # The file is isolated
            prefix_removed_filename: str = self._remove_prefix(
                path_to_file)
            if ext in ['.cpp', '.c', 'cxx']:
                shutil.copy(path_to_file, os.path.join(
                    self.output_dir, student_name, 'src', prefix_removed_filename))

            elif ext in ['.hpp', '.h']:
                shutil.copy(path_to_file, os.path.join(
                    self.output_dir, student_name, 'include', prefix_removed_filename))

            elif ext in ['.txt', '.md', '.pdf', '.word', '.tex', '.exe', '.out', '.bin', '']:
                shutil.copy(path_to_file, os.path.join(
                    self.output_dir, student_name, prefix_removed_filename))
            else:
                return False
        else:
            if ext in ['.cpp', '.c']:
                shutil.copy(path_to_file, os.path.join(
                    self.output_dir, student_name, 'src'))

            elif ext in ['.hpp', '.h']:
                shutil.copy(path_to_file, os.path.join(
                    self.output_dir, student_name, 'include'))

            elif ext in ['.txt', '.md', '.pdf', 'rtf', '.tex', '.exe', '.out', '.bin', 'doc', 'docx', 'html','']:
                shutil.copy(path_to_file, os.path.join(
                    self.output_dir, student_name))
            else:
                return False
        return True

    def _filter_process_dir(self, student_name: str, path_to_dir: str) -> bool:
        """Process a directory

        Args:
            student_name (str): student's name 5xxxxxxxxxxxNAME
            path_to_dir (str): path to a directory

        Returns:
            bool: If the submission is of correct format(has CMakeLists.txt)
        """
        if self.keep_output and os.path.exists(os.path.join(self.output_dir, student_name)):
            logging.info('Keeping {}'.format(os.path.join(self.output_dir, student_name)))
            return True
        # Check if the submission is of correct format
        ret: List[int] = self._test_submission_format(path_to_dir)

        if ret[1] > 0 or self.keep_file_structure > 0:  # Is of correct format, 
            self._clean_dir(os.path.join(self.output_dir, student_name))
            if ret[0] > 0:  # The entire folder is compressed and of correct format
                path_to_dir = os.path.join(
                    path_to_dir, [f for f in os.listdir(path_to_dir) if ((not f.startswith('.')) and (not f.startswith('__MACOSX')))][0])

            content_list = [f for f in os.listdir(
                path_to_dir) if not f.startswith('.')]
            for content in content_list:
                if (os.path.isdir(os.path.join(path_to_dir, content))):
                    shutil.copytree(os.path.join(path_to_dir, content), os.path.join(
                        self.output_dir, student_name, content))
                else:
                    shutil.copyfile(os.path.join(path_to_dir, content), os.path.join(
                        self.output_dir, student_name, content))
            return True
        else:  # Incorrect format
            for path_to_dir, _, filenames in os.walk(path_to_dir):
                for filename in filenames:
                    if filename[0] != '.':
                        self._filter_process_plain(
                            student_name, os.path.join(path_to_dir, filename))
            return False

    def _filter_all(self):
        """Apply filters to submission
        """
        pbar = tqdm(self.target_mapping.keys())
        for name in pbar:
            pbar.set_description('Processing {}'.format(name))
            for target in self.target_mapping[name]:
                if os.path.isdir(target):
                    self._filter_process_dir(name, target)
                else:
                    # For students who submitted seperated files, prefix must be removed
                    self._filter_process_plain(
                        name, target, remove_prefix=True)

    def run(self):
        """Run the filter
        """
        self._create_output_dir()
        self._preprocess()
        self._map_submission()
        self._create_alldir()
        self._filter_all()
        logging.info("The unprocessed files are:")
        for target_name in self.failed_targets:
            print('>', target_name)

    def __call__(self):
        self.run()



import subprocess
import os
from typing import Dict, List, Tuple
from tqdm import tqdm


class AutoBuilder:
    """Build submissions
    """
    shell_executable: str = '/bin/zsh'

    def __init__(self, args, shell_executable: str='/bin/zsh'):
        self.task_dir: str = args.output_dir
        self.original_working_dir: str = os.getcwd()
        self.task_list: List[str] = list()
        self.compiler_output: Dict[str, Tuple[int, str]] = dict()
        # Fullfill the build instruction
        self.compiler_command = args.command
        self.shell_executable = shell_executable

    @property
    def _build_instruction(self) -> List[str]:
        return [self.shell_executable, '-c', self.compiler_command]

    @property
    def _cmake_instructions(self) ->List[List[str]]:
        return [
        [self.shell_executable, '-c', 'cmake ./'], [self.shell_executable, '-c', 'make']]
    
    def _list_tasks(self):
        """List the output directory for unprocessed tasks
        """
        for target in [f for f in os.listdir(self.task_dir) if not f.startswith('.')]:
            if target[0] != '.':
                self.task_list.append(target)

    def _proble_cmake(self) -> bool:
        """Check if the directory contains CMakeLists.txt

        Returns:
            bool: exist -> True inexist -> False
        """
        content_list = [f for f in os.listdir(
            os.getcwd()) if not f.startswith('.')]
        if 'CMakeLists.txt' in content_list:
            return True
        else:
            return False

    def _probe_makefile(self) -> bool:
        """Check if the directory contains Makefile

        Returns:
            bool: exist -> True inexist -> False
        """
        content_list = [f for f in os.listdir(
            os.getcwd()) if not f.startswith('.')]
        if 'Makefile' in content_list:
            return True
        else:
            return False

    def _build_executable_cmake(self, retcode_tmp, output_tmp):
        ret = subprocess.run(
                    self._cmake_instructions[0], shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        retcode_tmp += ret.returncode
        output_tmp += str(ret.stderr + ret.stdout, encoding='UTF-8')

        return self._build_executable_make(retcode_tmp, output_tmp)
       
    def _build_executable_make(self, retcode_tmp, output_tmp):
        ret = subprocess.run(
                    self._cmake_instructions[1], shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        retcode_tmp += ret.returncode
        output_tmp += str(ret.stderr + ret.stdout, encoding='UTF-8')
        
        if retcode_tmp != 0:
            return self._build_executable_cmd(0, output_tmp + '\nMakefile failed, fallback to command')
        return retcode_tmp, output_tmp
    
    def _build_executable_cmd(self, retcode_tmp, output_tmp):
        ret = subprocess.run(
                    self._build_instruction, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        retcode_tmp += ret.returncode
        output_tmp += str(ret.stderr + ret.stdout, encoding='UTF-8')
        return retcode_tmp, output_tmp

        
    def _build_executable(self):
        """Build executable for all submissions
        """
        pbar = tqdm(self.task_list)
        os.chdir(self.original_working_dir)

        for task_name in pbar:
            retcode_tmp = 0
            output_tmp = ''

            pbar.set_description('Building {}'.format(task_name))
            # Change current warding directory
            os.chdir(os.path.join(self.task_dir, task_name))

            if self._proble_cmake():  # CMakeLists.txt is found
                retcode_tmp, output_tmp = self._build_executable_cmake(retcode_tmp, output_tmp)

            elif self._probe_makefile():  # Makefile is found
                retcode_tmp, output_tmp = self._build_executable_make(retcode_tmp, output_tmp)

            else:
                retcode_tmp, output_tmp = self._build_executable_cmd(retcode_tmp, output_tmp)

            self.compiler_output[task_name] = (retcode_tmp, output_tmp)

            # Change back
            os.chdir(self.original_working_dir)

    def run(self):
        """Run builder
        """
        self._list_tasks()
        self._build_executable()

    def __call__(self):
        self.run()



from setuptools import setup

requirements = ['rarfile==4.0', 'tqdm==4.56.0', 'replit==1.2.11', 'typing-extensions==3.7.4.3','coloredlogs==15.0.1']

setup(name="canvas-cppgrader",
      version="1.1.2",
      author="davidliyutong",
      keywords=["canvas", "grader", "cpp"],
      author_email="davidliyutong@sjtu.edu.cn",
      description="A toolkit to grade C/C++ submissions from canvas",
      long_description=open('README.rst').read(),
      license="MIT Licence",
      packages=["cppgrader", "cppgrader/components", "cppgrader/tools", "cppgrader/tools/antiplag"],
      python_requires=">=3.6",
      install_requires=requirements,
      entrypoints={'console_scripts': ['cppgrader = cppgrader.app:main']})



import subprocess
import os
from typing import List, Dict
import sys
import time
import replit

class ManualGrader:
    __doc__ = "Use '<' or ',' for previous submission\n" \
              "    '>' or '.' for next submission\n" \
              "'grade \{score1\} \{score2\}' or 'g \{score1\} \{score2\}' to grade\n" \
              "Type the name of executable to execute\n" \
              "'ls' and 'cd' commands are available\n" \
              "'save' to save and 'quit' to quit\n" \
              "\nMethods:\n\n\trun()"
    path_to_output: str = ''

    def __init__(self, path_to_output: str = './output', path_to_report: str = './GRADES.csv', dimension: List[str] = ['Answer', 'Interface']):
        self.path_to_output: str = path_to_output
        self.path_to_grade_report: str = path_to_report
        self.dimension: List[str] = dimension
        self.grades: Dict[str, List[float]] = dict()
        self._restor_grades()

    def _restor_grades(self):
        """Restore the grades from self.path_to_grade_report
        """
        if os.path.exists(self.path_to_grade_report):
            with open(self.path_to_grade_report, 'r') as f:
                for line in f.readlines()[1:]: # Discard the first line
                    line_tokenized = line.split(',')
                    self.grades[line_tokenized[0]] = [float(score) for score in line_tokenized[1:]]

    @property
    def _output_path_exist(self) -> bool:
        """Judge if the output path exist

        Returns:
            bool: Result
        """
        return os.path.exists(self.path_to_output)

    @property
    def _output_name_list(self) -> List[str]:
        """Return a list of output

        Returns:
            List[str]: ['5xxxxxxxxxxNAME1', '5xxxxxxxxxxNAME2,...']
        """
        if os.path.exists(self.path_to_output):
            return [f for f in os.listdir(self.path_to_output) if not f.startswith('.')]
        else:
            return []

    def _execute(self, task_name: str, executable_name: str):
        path_to_exe = os.path.join(
            self.path_to_output, task_name, executable_name)
        p = subprocess.Popen(['/bin/zsh', '-c', path_to_exe],
                             stdin=sys.stdin, stderr=subprocess.PIPE, stdout=sys.stdout)
        while p.poll() is None:
            time.sleep(0.1)

    def _make_grade(self, task_name, grade):
        try:
            self.grades[task_name] = [float(score) for score in grade]
        except ValueError:
            print('[ Error ] Value error')

    def _list_submission_dir(self, path_to_dir):
        if os.path.exists(path_to_dir):
            content_list = os.listdir(path_to_dir)
            print('>', '\n> '.join(content_list))
            return content_list
        else:
            print('[ Error ] Directory does not exist')
            return []

    def run(self):
        if not self._output_path_exist:
            path = input('Input output path:')  # Get output_path
            if os.path.exists(path):
                self.path_to_output = path

        task_list: List[str] = self._output_name_list
        task_list_len: List = len(task_list)

        idx: int = 0 # index of submission
        dir_tmp = '' # Tmp directory relate to submission, used by 'cd' and 'ls'

        while True:

            # Interfering with keyboard input
            print('\n[ Info ] Listing submission: {}'.format(task_list[idx]))

            content_list = self._list_submission_dir(
                os.path.join(self.path_to_output, task_list[idx], dir_tmp)) # List content of submission

            if task_list[idx] in self.grades.keys(): # Print current grades
                print('Current Grades: ', ' '.join([str(score) for score in self.grades[task_list[idx]]]))
            
            cmd: List[str] = input('% ').split(' ') # Get keyboard input
            if cmd[0] == '>' or cmd[0] == '.' or cmd[0] == '\x1b[C' or cmd[0] == '\x1b[B': # Move to next submission
                if idx < task_list_len - 1:
                    idx += 1 # increase index
                    dir_tmp = '' # reset Tmp directory
                    replit.clear()# Clear terminal output
                    continue
                else:
                    print('[ Info ] End of submissions')
                    continue

            if cmd[0] == '<' or cmd[0] == ',' or cmd[0] == '\x1b[D' or cmd[0] == '\x1b[A': # Move to previous submission
                if idx > 0:
                    idx -= 1 # decrease index
                    dir_tmp = '' # reset Tmp directory
                    replit.clear()# Clear terminal output
                    continue
                else:
                    print('[ Info ] Begin of submissions')
                    continue

            if cmd[0] in content_list or cmd[0] == '': # Execute submission
                if cmd[0] == '': # Name of executable is 'main' by default
                    cmd[0] = 'main'
                if os.path.isfile(os.path.join(self.path_to_output, task_list[idx], dir_tmp, cmd[0])):
                    print('\n\n---- Begin ----\n')
                    self._execute(task_list[idx], os.path.join(dir_tmp, cmd[0]))
                    print('\n---- End ----\n\n')

                else:
                    print('[ Error ] Not a file')
                continue

            if cmd[0] == 'cd': # Usage similar to 'cd'
                if len(cmd) > 1:
                    if os.path.exists(os.path.join(self.path_to_output, task_list[idx], dir_tmp, cmd[1])):
                        dir_tmp = os.path.join(dir_tmp, cmd[1])
                    else:
                        print('[ Error ] Directory does not exist')
                else:
                    print('[ Error ] Wrong argument')
                continue

            if cmd[0] == 'ls': # Usage similar to 'ls'
                if len(cmd) > 1:
                    self._list_submission_dir(os.path.join(
                        self.path_to_output, task_list[idx], dir_tmp, cmd[1]))
                else:
                    continue

            if cmd[0] == 'grade' or cmd[0] == 'g': # Grade, can accept multiple input score
                if len(cmd) > 1:
                    self._make_grade(task_list[idx], cmd[1:])
                    if idx < task_list_len - 1: idx += 1 # Next submission
                    replit.clear()# Clear terminal output
                    continue
                else:
                    print('[ Error ] Insufficient argument')

            if cmd[0] == 'save' or cmd[0] == 's':
                self.save()

            if cmd[0] == 'quit' or cmd[0] == 'q':
                print('[ Info ] Quitting...')
                self.save()
                break

            if cmd[0] == 'help' or cmd[0] == 'h':
                print(self.__doc__)

    def save(self):
        print('[ Info ] Saving grade report to: {}'.format(self.path_to_grade_report))
        with open(self.path_to_grade_report, 'w') as f:
            f.writelines(['Name,', ','.join(self.dimension), '\n'])
            for task_name in self.grades.keys():
                f.writelines([task_name, ',', ','.join([str(score)
                                                        for score in self.grades[task_name]]),'\n'])


    def __call__(self, *args, **kwargs):
        try:
            self.run(*args, **kwargs)
        except:
            self.save()


if __name__ == '__main__':
    App = ManualGrader()
    App()
